@@ -8,7 +8,7 @@ import git2net
 import shutil
 from pathlib import Path
 import stat
-
+import numpy
 from .utility import Utility
 
 class Version():
@@ -48,6 +48,10 @@ class Version():
         Extracting version data from a local repository and storing them in a mysql data base.
     generate_version_pandas_tables(repo, data_root_dir)
         Extracting edits and commits in a pandas table.
+    define_unknown_users(user_list, data_root_dir)
+        Define unknown users in commits pandas table.
+    get_unknown_users(data_root_dir)
+        Get all unknown users in from commits.
     get_version(data_root_dir, filename=VERSION_COMMITS)
         Get the generated pandas table.
 
@@ -223,6 +227,8 @@ class Version():
 
         pd_edits.rename(columns=Version.EDIT_RENAMING_COLUMNS, inplace = True)
 
+        pd_edits = pd_edits.fillna(value=0).astype({'total_added_lines' : 'int', 'total_removed_lines' : 'int'})
+
         # Embed author uuid
         users_ids = Utility.get_users_ids(data_root_dir)
         pd_commits['author'] = ""
@@ -238,6 +244,9 @@ class Version():
                                                                         users_ids, data_root_dir)   
                     pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'author'] = author_id   
                     pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'committer'] = committer_id 
+                    if (author_id is None) and (committer_id is None):
+                        pd_commits.loc[pd_commits.commit_sha == row.commit_sha, 'unknown_user'] = row.committer_name
+
             else:
                 commit_sha = pd_commits[pd_commits.committer_name == commiter_name].iloc[0].commit_sha 
                 author_id = Utility.extract_author_data_from_commit(repo, commit_sha, 
@@ -246,8 +255,21 @@ class Version():
                                                                     users_ids, data_root_dir)   
                 pd_commits.loc[pd_commits.committer_name == commiter_name, 'author'] = author_id   
                 pd_commits.loc[pd_commits.committer_name == commiter_name, 'committer'] = committer_id 
+                if (author_id is None) and (committer_id is None):
+                    pd_commits.loc[pd_commits.committer_name == commiter_name, 'unknown_user'] = commiter_name 
         pd_commits.drop(['committer_name'], axis=1, inplace=True)  
 
+        users = Utility.get_users(data_root_dir)
+        if "unknown_user" in pd_commits:
+            unknown_user_commits = pd_commits.loc[pd_commits.unknown_user.notna()]
+            unknown_users = unknown_user_commits.unknown_user.unique()
+            for unknown_user in unknown_users:
+                if not users.empty:
+                    for index, row in users.iterrows():
+                        if (row["email"] == unknown_user) or (row["name"] == unknown_user) or (row["login"] == unknown_user):
+                            pd_commits.loc[pd_commits.unknown_user == unknown_user, 'author'] = row["anonym_uuid"]
+                            pd_commits.loc[pd_commits.unknown_user == unknown_user, 'committer'] = row["anonym_uuid"]
+                            pd_commits.loc[pd_commits.unknown_user == unknown_user, 'unknown_user'] = numpy.NaN
         # Extract Tags
         pd_commits['tag'] = ""
         tags = repo.get_tags()
@@ -280,6 +302,66 @@ class Version():
         with open(pd_branches_file, "wb") as f:
             pickle.dump(pd_Branches, f)          
 
+    @staticmethod
+    def define_unknown_users(user_dict, data_root_dir):
+        """
+        define_unknown_users(user_dict, data_root_dir)
+
+        Define unknown users in commits pandas table.
+
+        Parameters
+        ----------
+        user_dict: dict
+            Dictionary which contains users. 
+        data_root_dir : str
+            Data root directory for the repository.
+
+        Notes
+        -----
+        Example User: {"unknown_user": "real user node id"}
+        If the real user node id does not exist in the users table then a new user will be created
+        
+        """
+        pd_commits = Version.get_version(data_root_dir)
+        if "unknown_user" in pd_commits:
+            unknown_user_commits = pd_commits.loc[pd_commits.unknown_user.notna()]
+            unknown_users = unknown_user_commits.unknown_user.unique()
+            for unknown_user in unknown_users:
+                uuid = Utility.define_unknown_user(user_dict,unknown_user,data_root_dir)
+                if uuid is not None:
+                    pd_commits.loc[pd_commits.unknown_user == unknown_user, 'author'] = uuid
+                    pd_commits.loc[pd_commits.unknown_user == unknown_user, 'committer'] = uuid
+                    pd_commits.loc[pd_commits.unknown_user == unknown_user, 'unknown_user'] = numpy.NaN
+
+            version_folder = Path(data_root_dir, Version.VERSION_DIR)
+            pd_commits_file = Path(version_folder, Version.VERSION_COMMITS)
+            with open(pd_commits_file, "wb") as f:
+                pickle.dump(pd_commits, f)
+        
+    @staticmethod
+    def get_unknown_users(data_root_dir):
+        """
+        get_unknown_users(data_root_dir)
+
+        Get all unknown users in from commits.
+
+        Parameters
+        ----------
+        data_root_dir : str
+            Data root directory for the repository.
+
+        Returns
+        -------
+        List
+            List of unknown user names
+        
+        """
+        pd_commits = Version.get_version(data_root_dir)
+        if "unknown_user" in pd_commits:
+            unknown_user_commits = pd_commits.loc[pd_commits.unknown_user.notna()]
+            unknown_users = unknown_user_commits.unknown_user.unique()
+            return list(unknown_users)
+
     @staticmethod
     def get_version(data_root_dir, filename=VERSION_COMMITS):
         """
